package com.activetuts.slider {
	
	import flash.display.*;
	import flash.events.*;
	import flash.geom.*;
	import com.activetuts.events.SliderEvent;
	
	public class ActiveSlider extends EventDispatcher {
		
		private var _track:Sprite;
		private var _grip:Sprite;
		private var _grabOffset:Point;
		
		public function ActiveSlider(track:Sprite, grip:Sprite) {
			_track = track;
			_grip = grip;
			if (_grip.parent != _track.parent) {
				throw new Error("The track and the grip Sprites are not in the same container.")
			}
			
			_grip.addEventListener(MouseEvent.MOUSE_DOWN, onDown);
			
			if (_grip.stage) {
				addStageListener();
			} else {
				_grip.addEventListener(Event.ADDED_TO_STAGE, onAddedToStage);
			}
		}
		
		private function onAddedToStage(e:Event):void {
			_grip.removeEventListener(Event.ADDED_TO_STAGE, onAddedToStage);
			addStageListener();
		}
		
		private function addStageListener():void {
			_grip.stage.addEventListener(MouseEvent.MOUSE_UP,   onUp);
		}
		
		
		private function onDown(e:MouseEvent):void {
			_grabOffset = new Point(e.localX, e.localY);
			_grip.addEventListener(Event.ENTER_FRAME, onFrame);
		}
		
		private function onUp(e:MouseEvent):void {
			_grip.removeEventListener(Event.ENTER_FRAME, onFrame);
		}
		
		private function onFrame(e:Event):void {
			_grip.x = _grip.parent.mouseX - _grabOffset.x;
			
			var gripBounds = _grip.getBounds(_grip.parent);
			var trackBounds = _track.getBounds(_grip.parent);
			
			if (gripBounds.x < trackBounds.x) {
				_grip.x = _track.x;
			} else if (gripBounds.right > trackBounds.right) {
				_grip.x = trackBounds.right - gripBounds.width
			}
			
			dispatchEvent(new SliderEvent(SliderEvent.CHANGE, this.value));
		}
		
		
		public function get value():Number {
			return (_grip.x - _track.x) / (_track.width - _grip.width);
		}
		
		
		
		
	}
	
}